from pyduinocli.commands.base import CommandBase
from pyduinocli.constants import commands
from pyduinocli.constants import flags


class ConfigCommand(CommandBase):
    """
    This class wraps the call to the :code:`config` command of :code:`arduino-cli`.
    """

    def __init__(self, base_args):
        CommandBase.__init__(self, base_args)
        self._base_args.append(commands.CONFIG)

    def dump(self):
        """
        Calls the :code:`config dump` command

        :return: The output of the related command
        :rtype: dict
        """
        return self._exec([commands.DUMP])

    def init(self, dest_dir=None, dest_file=None, overwrite=None):
        """
        Calls the :code:`config init` command

        :param dest_dir: The directory where to save the file
        :type dest_dir: str or NoneType
        :param dest_file: A path where the file will be saved
        :type dest_file: str or NoneType
        :param overwrite: Overwrites an existing file
        :type overwrite: bool or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.INIT]
        if dest_dir:
            args.extend([flags.DEST_DIR, CommandBase._strip_arg(dest_dir)])
        if dest_file:
            args.extend([flags.DEST_FILE, CommandBase._strip_arg(dest_file)])
        if overwrite is True:
            args.append(flags.OVERWRITE)
        return self._exec(args)

    def add(self, setting_name, values):
        """
        Calls the :code:`config add` command

        :param setting_name: The name of the setting to add
        :type setting_name: str
        :param values: The values to add to the setting
        :type values: list
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.ADD, CommandBase._strip_arg(setting_name)]
        args.extend(CommandBase._strip_args(values))
        return self._exec(args)

    def delete(self, setting_name):
        """
        Calls the :code:`config delete` command

        :param setting_name: The name of the setting to delete
        :type setting_name: str
        :return: The output of the related command
        :rtype: dict
        """
        return self._exec([commands.DELETE, CommandBase._strip_arg(setting_name)])

    def remove(self, setting_name, values):
        """
        Calls the :code:`config remove` command

        :param setting_name: The name of the setting from which values will be removed
        :type setting_name: str
        :param values: The list of values to remove
        :type values: list
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.REMOVE, CommandBase._strip_arg(setting_name)]
        args.extend(CommandBase._strip_args(values))
        return self._exec(args)

    def set(self, setting_name, values):
        """
        Calls the :code:`config set` command

        :param setting_name: The name of the setting to set
        :type setting_name: str
        :param values: The list of values to set
        :type values: list
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.SET, CommandBase._strip_arg(setting_name)]
        args.extend(CommandBase._strip_args(values))
        return self._exec(args)



from pyduinocli.commands.base import CommandBase
from pyduinocli.constants import commands
from pyduinocli.constants import flags


class BoardCommand(CommandBase):
    """
    This class wraps the call to the :code:`board` command of :code:`arduino-cli`
    """

    def __init__(self, base_args):
        CommandBase.__init__(self, base_args)
        self._base_args.append(commands.BOARD)

    def attach(self, port=None, fqbn=None, sketch_path=None, discovery_timeout=None, protocol=None, board_options=None):
        """
        Calls the :code:`board attach` command.

        :param port: Upload port address, e.g.: COM3 or /dev/ttyACM2
        :type port: str or NoneTYpe
        :param fqbn: Fully Qualified Board Name, e.g.: arduino:avr:uno
        :type fqbn: str or NoneTYpe
        :param sketch_path: The path of the sketch to attach to the board
        :type sketch_path: str or NoneTYpe
        :param discovery_timeout: Max time to wait for port discovery, e.g.: 30s, 1m (default 5s)
        :type discovery_timeout: str or NoneTYpe
        :param protocol: Upload port protocol, e.g: serial
        :type protocol: str or NoneTYpe
        :param board_options: Board options
        :type board_options: dict or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.ATTACH]
        if port:
            args.extend([flags.PORT, CommandBase._strip_arg(port)])
        if protocol:
            args.extend([flags.PROTOCOL, CommandBase._strip_arg(protocol)])
        if fqbn:
            args.extend([flags.FQBN, CommandBase._strip_arg(fqbn)])
        if sketch_path:
            args.append(CommandBase._strip_arg(sketch_path))
        if discovery_timeout:
            args.extend([flags.DISCOVERY_TIMEOUT, CommandBase._strip_arg(discovery_timeout)])
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)

    def details(self, fqbn, full=None, list_programmers=None, board_options=None):
        """
        Calls the :code:`board details` command.

        :param fqbn: The fqbn of the board
        :type fqbn: str
        :param full: Show full board details
        :type full: bool or NoneTYpe
        :param list_programmers: Show list of available programmers
        :type list_programmers: bool or NoneType
        :param board_options: Board options
        :type board_options: dict or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.DETAILS, flags.FQBN, CommandBase._strip_arg(fqbn)]
        if full is True:
            args.append(flags.FULL)
        if list_programmers is True:
            args.append(flags.LIST_PROGRAMMERS)
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)

    def list(self, discovery_timeout=None, watch=None, board_options=None, fqbn=None):
        """
        Calls the :code:`board list` command.

        :param discovery_timeout: Max time to wait for port discovery, e.g.: 30s, 1m (default 1s)
        :type discovery_timeout: str or NoneType
        :param watch: Command keeps running and prints list of connected boards whenever there is a change. Added to pyduinocli for completion but won't actually return, use a loop instead
        :type watch: bool or NoneTYpe
        :param fqbn: Fully Qualified Board Name, e.g.: arduino:avr:uno
        :type fqbn: str or NoneType
        :param board_options: Dict of board options (option_name:value)
        :type board_options: dict or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.LIST]
        if discovery_timeout:
            args.extend([flags.DISCOVERY_TIMEOUT, CommandBase._strip_arg(discovery_timeout)])
        if watch is True:
            args.append(flags.WATCH)
        if fqbn is not None:
            args.extend([flags.FQBN, CommandBase._strip_arg(fqbn)])
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)

    def listall(self, boardname=None, show_hidden=None):
        """
        Calls the :code:`board listall` command.

        :param boardname: The name of the board, all board will be returned if left unset (or None)
        :type boardname: str or NoneType
        :param show_hidden: Show also boards marked as 'hidden' in the platform
        :type show_hidden: bool or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.LISTALL]
        if boardname:
            args.append(CommandBase._strip_arg(boardname))
        if show_hidden is True:
            args.append(flags.SHOW_HIDDEN)
        return self._exec(args)

    def search(self, boardname=None, show_hidden=None):
        """
        Calls the :code:`board search` command.

        :param boardname: The name of the board
        :type boardname: str or NoneType
        :param show_hidden: Show also boards marked as 'hidden' in the platform
        :type show_hidden: bool or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.SEARCH]
        if boardname:
            args.append(CommandBase._strip_arg(boardname))
        if show_hidden is True:
            args.append(flags.SHOW_HIDDEN)
        return self._exec(args)



from pyduinocli.commands.base import CommandBase
from pyduinocli.constants import commands


class CacheCommand(CommandBase):
    """
    This class wraps the call to the :code:`cache` command of :code:`arduino-cli`.
    """

    def __init__(self, base_args):
        CommandBase.__init__(self, base_args)
        self._base_args.append(commands.CACHE)

    def clean(self):
        """
        Calls the :code:`cache clean` command

        :return: The output of the related command
        :rtype: dict
        """
        return self._exec([commands.CLEAN])



ADDITIONAL_URLS = '--additional-urls'
CONFIG_FILE = '--config-file'
FORMAT = '--format'
LOG_FILE = '--log-file'
LOG_FORMAT = '--log-format'
LOG_LEVEL = '--log-level'
BUILD_CACHE_PATH = '--build-cache-path'
BUILD_PATH = '--build-path'
BUILD_PROPERTY = '--build-property'
FQBN = '--fqbn'
OUTPUT = '--output'
UPLOAD = '--upload'
PREPROCESS = '--preprocess'
SHOW_PROPERTIES = '--show-properties'
WARNINGS = '--warnings'
DEST_DIR = '--dest-dir'
VERIFY = '--verify'
INPUT = '--input'
PORT = '--port'
UPDATABLE = '--updatable'
NAMES = '--names'
ALL = '--all'
DAEMONIZE = '--daemonize'
LIBRARIES = '--libraries'
OPTIMIZE_FOR_DEBUG = '--optimize-for-debug'
INTERPRETER = '--interpreter'
FULL = '--full'
PROGRAMMER = '--programmer'
OUTPUT_DIR = '--output-dir'
NO_DESCRIPTION = '--no-description'
INPUT_DIR = '--input-dir'
LIST_PROGRAMMERS = '--list-programmers'
RUN_POST_INSTALL = '--run-post-install'
SKIP_POST_INSTALL = '--skip-post-install'
INCLUDE_BUILD_DIR = '--include-build-dir'
SHOW_OUTDATED = '--show-outdated'
INPUT_FILE = '--input-file'
WATCH = '--watch'
SHOW_HIDDEN = '--show-hidden'
CLEAN = '--clean'
EXPORT_BINARIES = '--export-binaries'
DEST_FILE = '--dest-file'
OVERWRITE = '--overwrite'
INFO = '--info'
GIT_URL = '--git-url'
ZIP_PATH = '--zip-path'
ONLY_COMPILATION_DATABASE = '--only-compilation-database'
LIBRARY = '--library'
DISCOVERY_TIMEOUT = '--discovery-timeout'
PROTOCOL = '--protocol'
NO_COLOR = '--no-color'
DEBUG = '--debug'
DEBUG_FILTER = '--debug-filter'
CONFIG = '--config'
DESCRIBE = '--describe'
QUIET = '--quiet'
BOARD_OPTIONS = '--board-options'
ENCRYPT_KEY = '--encrypt-key'
KEYS_KEYCHAIN = '--keys-keychain'
SIGN_KEY = '--sign-key'
DEBUG_FILE = '--debug-file'
DUMP_PROFILE = '--dump-profile'
PROFILE = '--profile'
NO_OVERWRITE = '--no-overwrite'
INSTALL_IN_BUILTIN_DIR = '--install-in-builtin-dir'
OMIT_RELEASES_DETAILS = '--omit-releases-details'



from pyduinocli.commands.base import CommandBase
from pyduinocli.constants import commands
from pyduinocli.constants import flags


class MonitorCommand(CommandBase):
    """
    This class wraps the call to the :code:`monitor` command of :code:`arduino-cli`.
    """

    def __init__(self, base_args):
        CommandBase.__init__(self, base_args)
        self._base_args.append(commands.MONITOR)

    def __call__(self, config=None, describe=None, discovery_timeout=None, fqbn=None, port=None, protocol=None,
                 quiet=None, board_options=None):
        """
        Calls the :code:`monitor` command

        :param config: Configuration of the port.
        :type config: str or NoneType
        :param describe: Show all the settings of the communication port.
        :type describe: bool or NoneType
        :param discovery_timeout: Max time to wait for port discovery, e.g.: 30s, 1m (default 5s)
        :type discovery_timeout: str or NoneType
        :param fqbn: Fully Qualified Board Name, e.g.: arduino:avr:uno
        :type fqbn: str or NoneType
        :param port: Upload port address, e.g.: COM3 or /dev/ttyACM2
        :type port: str or NoneType
        :param protocol: Upload port protocol, e.g: serial
        :type protocol: str or NoneType
        :param quiet: Run in silent mode, show only monitor input and output.
        :type quiet: bool or NoneType
        :param board_options: Board options
        :type board_options: dict or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = []
        if config:
            args.extend([flags.CONFIG, CommandBase._strip_arg(config)])
        if describe is True:
            args.append(flags.DESCRIBE)
        if discovery_timeout:
            args.extend([flags.DISCOVERY_TIMEOUT, CommandBase._strip_args(discovery_timeout)])
        if fqbn:
            args.extend([flags.FQBN, CommandBase._strip_arg(fqbn)])
        if port:
            args.extend([flags.PORT, CommandBase._strip_arg(port)])
        if protocol:
            args.extend([flags.PROTOCOL, CommandBase._strip_arg(protocol)])
        if quiet is True:
            args.append(flags.QUIET)
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)



from pyduinocli.commands.base import CommandBase
from pyduinocli.constants import commands
from pyduinocli.constants import flags


class LibCommand(CommandBase):
    """
    This class wraps the call to the :code:`lib` command of :code:`arduino-cli`.
    """

    def __init__(self, base_args):
        CommandBase.__init__(self, base_args)
        self._base_args.append(commands.LIB)

    def deps(self, library):
        """
        Calls the :code:`lib deps` command

        :param library: The name of the library for dependency checking
        :type library: str
        :return: The output of the related command
        :rtype: dict
        """
        return self._exec([commands.DEPS, CommandBase._strip_arg(library)])

    def download(self, downloads):
        """
        Calls the :code:`lib download` command

        :param downloads: A list of libraries to download
        :type downloads: list
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.DOWNLOAD]
        args.extend(CommandBase._strip_args(downloads))
        return self._exec(args)

    def install(self, libraries=None, git_urls=None, zip_paths=None, no_overwrite=None, install_in_builtin_dir=None):
        """
        Calls the :code:`lib install` command

        :param libraries: A list of libraries to install
        :type libraries: list or NoneType
        :param git_urls: A list of git repositories containing libraries to install
        :type git_urls: list or NoneType
        :param zip_paths: A list of paths to zip files to install
        :type zip_paths: list or NoneType
        :param no_overwrite: Do not overwrite already installed libraries.
        :type no_overwrite: bool or NoneType
        :param install_in_builtin_dir: Install libraries in the IDE-Builtin directory
        :type install_in_builtin_dir: bool or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.INSTALL]
        if libraries:
            args.extend(CommandBase._strip_args(libraries))
        if git_urls:
            args.append(flags.GIT_URL)
            args.extend(CommandBase._strip_args(git_urls))
        if zip_paths:
            args.append(flags.ZIP_PATH)
            args.extend(CommandBase._strip_args(zip_paths))
        if no_overwrite is True:
            args.append(flags.NO_OVERWRITE)
        if install_in_builtin_dir is True:
            args.append(flags.INSTALL_IN_BUILTIN_DIR)
        return self._exec(args)

    def list(self, all=None, updatable=None, fqbn=None, board_options=None):
        """
        Calls the :code:`lib list` command

        :param all: Includes built-in libraries
        :type all: bool or NoneType
        :param updatable: Only shows libraries that are not up to date
        :type updatable: bool or NoneType
        :param fqbn: Shows libraries for the specified board
        :type fqbn: str or NoneType
        :param board_options: Board options
        :type board_options: dict or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.LIST]
        if all is True:
            args.append(flags.ALL)
        if updatable is True:
            args.append(flags.UPDATABLE)
        if fqbn is not None:
            args.extend([flags.FQBN, CommandBase._strip_arg(fqbn)])
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)

    def search(self, keywords=None, names=None, omit_releases_details=None):
        """
        Calls the :code:`lib search` command

        :param keywords: A list of keywords to use to search, if None, all libraries will show up
        :type keywords: list or NoneType
        :param omit_releases_details: Omit library details far all versions except the latest (produce a more compact JSON output).
        :type omit_releases_details: bool or NoneType
        :param names: Only shows libraries names
        :type names: bool or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.SEARCH]
        if names is True:
            args.append(flags.NAMES)
        if omit_releases_details is True:
            args.append(flags.OMIT_RELEASES_DETAILS)
        if keywords is None:
            keywords = []
        if keywords:
            args.extend(CommandBase._strip_args(keywords))
        return self._exec(args)

    def uninstall(self, uninstalls):
        """
        Calls the :code:`lib uninstall` command

        :param uninstalls: A list of libraries to uninstall
        :type uninstalls: list
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.UNINSTALL]
        args.extend(CommandBase._strip_args(uninstalls))
        return self._exec(args)

    def update_index(self):
        """
        Calls the :code:`lib update-index` command

        :return: The output of the related command
        :rtype: dict
        """
        return self._exec([commands.UPDATE_INDEX])

    def upgrade(self, upgrades=None):
        """
        Calls the :code:`lib upgrade` command

        :param upgrades: A list of libraries to upgrade, if None, all libraries will be upgraded
        :type upgrades: list or NoneType
        :return: The output of the related command
        :rtype: dict
        """
        if not upgrades:
            upgrades = []
        args = [commands.UPGRADE]
        args.extend(CommandBase._strip_args(upgrades))
        return self._exec(args)

    def examples(self, library, fqbn=None, board_options=None):
        """
        Calls the :code:`lib examples` command

        :param library: The name of the library
        :type library: str
        :param fqbn: The board FQBN
        :type fqbn: str or NoneType
        :param board_options: Board options
        :type board_options: dict or NoneTYpe
        :return: The output of the related command
        :rtype: dict
        """
        args = [commands.EXAMPLES, CommandBase._strip_arg(library)]
        if fqbn is not None:
            args.extend([flags.FQBN, CommandBase._strip_arg(fqbn)])
        if board_options:
            for option_name, option_value in board_options.items():
                option = "%s=%s" % (CommandBase._strip_arg(option_name), CommandBase._strip_arg(option_value))
                args.extend([flags.BOARD_OPTIONS, option])
        return self._exec(args)

